package dnanalytics.view.tools.callVariants;

import dnanalytics.utils.FileManager;
import dnanalytics.utils.Settings;
import dnanalytics.view.DNAMain;
import dnanalytics.view.tools.Tool;
import dnanalytics.worker.Haplotype;
import dnanalytics.worker.Worker;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.TextField;
import javafx.scene.layout.VBox;

public class CallVariants implements Tool {

    @FXML
    private static ResourceBundle resources = DNAMain.getResources();
    @FXML
    private URL location;
    @FXML
    private static TextField dbsnp;
    @FXML
    private static TextField hapmap;
    @FXML
    private static TextField input;
    @FXML
    private static TextField mills;
    @FXML
    private static TextField omni;
    @FXML
    private static TextField output;
    @FXML
    private static CheckBox recalibrate;
    @FXML
    private static VBox trainingDatasets;

    @FXML
    void initialize() {

        recalibrate.selectedProperty().addListener(
                new ChangeListener<Boolean>() {
            @Override
            public void changed(ObservableValue<? extends Boolean> ov, Boolean t,
                    Boolean t1) {
                trainingDatasets.setDisable(!t1);
            }
        });

        String value = Settings.getProperty("call.dbsnp");
        dbsnp.setText(value != null ? value : "");
        value = Settings.getProperty("call.mills");
        mills.setText(value == null ? "" : value);
        value = Settings.getProperty("call.omni");
        omni.setText(value == null ? "" : value);
        value = Settings.getProperty("call.hapmap");
        hapmap.setText(value == null ? "" : value);
    }

    @FXML
    void selectInput(ActionEvent event) {
        FileManager.setOpenFile(FileManager.SAM_BAM_DESCRIPTION,
                FileManager.SAM_BAM_DESCRIPTION, FileManager.SAM_BAM_FILTERS, input);
    }

    @FXML
    void selectOutput(ActionEvent event) {
        FileManager.saveVCF(output);
    }

    @FXML
    void selectDbsnp(ActionEvent event) {
        File f = FileManager.openVCF(dbsnp);
        if (f != null) {
            Settings.setProperty("call.dbsnp", f.getAbsolutePath());
        }
    }

    @FXML
    void selectHapmap(ActionEvent event) {
        File f = FileManager.openVCF(hapmap);
        if (f != null) {
            Settings.setProperty("call.hapmap", f.getAbsolutePath());
        }
    }

    @FXML
    void selectMills(ActionEvent event) {
        File f = FileManager.openVCF(mills);
        if (f != null) {
            Settings.setProperty("call.mills", f.getAbsolutePath());
        }
    }

    @FXML
    void selectOmni(ActionEvent event) {
        File f = FileManager.openVCF(omni);
        if (f != null) {
            Settings.setProperty("call.omni", f.getAbsolutePath());
        }
    }

    @Override
    public Node getView() {
        try {
            return FXMLLoader.load(getClass().getResource("CallVariants.fxml"),
                    DNAMain.getResources());
        } catch (IOException ex) {
            return null;
        }
    }

    private boolean checkParameters() {
        // Check for params
        // These files must exist: genome, tempDir, dbsnp, input
        // These string must not be empty: output
        // If recalibration, these files must exist: hapmap, omni, mills
        if (Settings.getGenome() == null || !Settings.getGenome().exists()) {
            System.err.println(resources.getString("no.genome"));
            return false;
        }
        if (!new File(dbsnp.getText()).exists()) {
            System.err.println(resources.getString("no.dbsnp"));
            return false;
        }
        if (!new File(Settings.getProperty("tempDir")).exists()) {
            System.err.println(resources.getString("no.temp"));
            return false;
        }
        if (output.getText().isEmpty()) {
            System.err.println(resources.getString("no.output"));
            return false;
        }
        if (!new File(input.getText()).exists()) {
            System.err.println(resources.getString("no.input"));
            return false;
        }
        if (recalibrate.isSelected()) {
            // Check for database
            if (!new File(mills.getText()).exists()) {
                System.err.println(resources.getString("no.mills"));
                return false;
            }
            if (!new File(hapmap.getText()).exists()) {
                System.err.println(resources.getString("no.hapmap"));
                return false;
            }
            if (!new File(omni.getText()).exists()) {
                System.err.println(resources.getString("no.omni"));
                return false;
            }
        }
        return true;
    }

    @Override
    public Worker getWorker() {

        return new Haplotype(
                Settings.getGenome().getAbsolutePath(), dbsnp.getText(),
                omni.getText(), hapmap.getText(), mills.getText(),
                output.getText(), input.getText(), recalibrate.isSelected(),
                Settings.getProperty("tempDir"));
    }

    @Override
    public String getTitle() {
        return resources.getString("call.title");
    }

    @Override
    public String getStyleID() {
        return "callVariants";
    }
}
