package dnanalytics.view.tools.dindel;

import dnanalytics.utils.FileManager;
import dnanalytics.utils.Settings;
import dnanalytics.view.DNAMain;
import dnanalytics.view.tools.Tool;
import dnanalytics.worker.Worker;
import dnanalytics.worker.WorkerScript;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.TextField;

/**
 *
 * @author uai
 */
public class Dindel implements Tool {

    // INPUT: BAM File
    public static TextField input;
    // OUTPUT: VCF File
    public static TextField output;
    // Sample name: string
    public static TextField name;
    private static ResourceBundle resources = DNAMain.
            getResources();

    /*
     * Métodos como controladora
     */
    public void selectInput() {
        FileManager.openBAM(input);
    }

    public void selectOutput() {
        FileManager.saveVCF(output);
    }

    /*
     * Métodos como herramienta
     */
    @Override
    public Node getView() {
        try {
            return FXMLLoader.load(getClass().getResource("dindel.fxml"), DNAMain.getResources());
        } catch (IOException ex) {
            return null;
        }
    }

    private boolean checkParameters() {
        // Checking parameters
        if (!new File(input.getText()).exists()) {
            System.err.println(resources.getString("no.input"));
            return false;
        }
        if (output.getText().isEmpty()) {
            System.err.println(resources.getString("no.output"));
            return false;
        }
        if (Settings.getGenome() == null || !Settings.getGenome().exists()) {
            System.err.println(resources.getString("no.genome"));
            return false;
        }
        return true;
    }

    @Override
    public Worker getWorker() {
        if (!checkParameters()) {
            return null;
        }
        return new WorkerScript() {
            // If output is /path/to/myfile.vcf, name will be myfile
            private String name;
            private File temp, windows, windows2, dindel;

            /**
             * Creates directory tree and initializes variables.
             */
            private void initializeSettings() {
                // + tempDir
                // | + name
                //   | - windows
                //   | - windows2
                name = new File(output.getText()).getName().replace(".vcf", "");
                temp = new File(Settings.getProperty("tempDir"), name);
                windows = new File(temp, "windows");
                windows2 = new File(temp, "windows2");
                temp.mkdir();
                windows.mkdir();
                windows2.mkdir();
                dindel = new File("software", "dindel");
            }

            /**
             * First step for standard dindel workflow: extraction of potential indels from the
             * input BAM file. The output are 2 files, named after sample name (name.libraries.txt
             * and name.variants.txt)
             */
            private void extractCandidatesFromBAM() {
                // Command appearance:
                // /home/uai/dindel/dindel
                //  --analysis getCIGARindels \
                //  --bamFile input.bam \
                //  --ref reference.fasta \
                //  --outputFile temp/name
                executeCommand(new File(dindel, "dindel")
                        + " --analysis getCIGARindels"
                        + " --bamFile " + input.getText()
                        + " --ref " + Settings.getGenome()
                        + " --outputFile " + new File(temp, name));
                // This will generate two files
                // temp/name/name.libraries.txt
                // temp/name/name.variants.txt

            }

            /**
             * Second step for standard dindel workflow: the indels obtained in stage 1 from the BAM
             * file are the candidate indels; they must be grouped into windows of ∼ 120 basepairs,
             * into a realign-window-file. The included Python script makeWindows.py will generate
             * such a file from the file with candidate indels inferred in the first stage. The
             * output are hundreds of files (temp/name/windows/window001.txt,
             * /temp/name/windows/window002.txt, /temp/name/windows/windowXXX.txt)
             */
            private void createRealignWindows() {
                // Appearance of the command
                //  python /home/uai/dindel/makeWindows.py \
                //    --inputFile temp/name/name.variants.txt \
                //    --windowFilePrefix windows/name_window \
                //    --numWindowsPerFile 1000
                executeCommand("python " + new File(dindel, "makeWindows.py")
                        + " --inputVarFile " + new File(temp, name + ".variants.txt")
                        + " --windowFilePrefix " + new File(windows, name + "_window")
                        + " --numWindowsPerFile 1000");
                // So:
                // temp/name/windows/name_window001.txt
                // temp/name/windows/name_window002.txt
                // temp/name/windows/name_window003.txt
                // temp/name/windows/name_window004.txt
                // ... and so on ...
            }

            /**
             * Third step for dindel standard workflow: for every window, Dindel will generate
             * candidate haplotypes from the candidate indels and SNPs it detects in the BAM file,
             * and realign the reads to these candidate haplotypes. The realignment step is the
             * computationally most intensive step.
             */
            private void realignToHaplotypes() {
                // Command appearance (we must do this for every window):
                // /home/uai/dindel/dindel
                //   --analysis indels \
                //   --doDiploid \
                //   --bamFile input.bam \
                //   --ref reference.fasta \
                //   --varFile temp/name/windows/name_window001.txt \
                //   --libFile temp/name/name.libraries.txt \
                //   --outputFile temp/name/windows2/name_windows001.txt \
                //   &>/dev/null
                // For this phase we will take into account progress.
                
                long start = System.currentTimeMillis();
                int i = 1;
                File[] files = windows.listFiles();
                int t = files.length;
                DateFormat df = new SimpleDateFormat("HH:mm:ss");
                outStream.println("ELAPSED TIME\tREMAINING\tWINDOWS");
                for (File file : files) {
                    updateMessage("Realigning " + i + " out of " + t + " windows");
                    executeCommand(new File(dindel, "dindel")
                            + " --analysis indels"
                            + " --doDiploid"
                            + " --bamFile " + input.getText()
                            + " --ref " + Settings.getGenome()
                            + " --varFile " + file
                            + " --libFile " + new File(temp, name + ".libraries.txt")
                            + " --outputFile " + new File(windows2, file.getName())
                            + " &>/dev/null");
                    long elapsed = System.currentTimeMillis() - start;
                    long remaining = (elapsed / i) * (t - i);
                    outStream.println(df.format(new Date(elapsed))
                            + "\t" + df.format(new Date(remaining))
                            + "\t" + i + "/" + t);
//                    outStream.print("Elapsed: " + df.format(new Date(elapsed)));
//                    outStream.println("\tRemaining: " + df.format(new Date(remaining)));
                    updateProgress(20 + 70 * i / t, 100);
                    i++;
                }
            }

            /**
             * Last step for dindel standard workflow: interpreting the output from Dindel and
             * produce indel calls and qualities in the VCF4 format.
             */
            private void mergeIndelsInVcf() {
                // Command appearance:
                //  python /home/uai/dindel/mergeOutputDiploid.py \
                //   --ref reference.fasta \
                //   --inputFiles fileList.txt \
                //   --outputFile output.vcf
                // where fileList contains a list of all windows2 files.
                File fileList = new File(temp, "fileList.txt");

                try (BufferedWriter bw = new BufferedWriter(new FileWriter(fileList))) {
                    for (File f : windows2.listFiles()) {
                        bw.write(f.getPath());
                        bw.newLine();
                    }
                } catch (IOException ex) {
                    Logger.getLogger(Dindel.class.getName()).log(Level.SEVERE, null, ex);
                }
                executeCommand("python " + new File(dindel, "mergeOutputDiploid.py")
                        + " --ref " + Settings.getGenome()
                        + " --inputFiles " + fileList
                        + " --outputFile " + output.getText());
            }

            @Override
            protected int start() {

                initializeSettings();

                updateMessage("Extracting candidate indels from BAM");
                updateProgress(0, 100);
                extractCandidatesFromBAM();

                updateMessage("Creating realignment windows");
                updateProgress(10, 100);
                createRealignWindows();

                updateMessage("Realigning");
                updateProgress(20, 100);
                realignToHaplotypes();

                updateMessage("Generating VCF");
                updateProgress(90, 100);
                mergeIndelsInVcf();
                updateProgress(1, 1);
                return 0;
            }
        };
    }

    @Override
    public String getTitle() {
        return "Dindel (Indels call)";
    }

    @Override
    public String getStyleID() {
        return "dindel";
    }
}
